@@ -29,6 +29,8 @@ class TransformationType(Enum):
     DATE_FUNCTION_AGGREGATION = "DATE_FUNCTION_AGGREGATION"
     CASE_AGGREGATION = "CASE_AGGREGATION"
     EXEC = "EXEC"
+    CONSTANT = "CONSTANT"
+    UNKNOWN = "UNKNOWN"
 
 
 @dataclass
@@ -38,6 +40,17 @@ class ColumnReference:
     table_name: str
     column_name: str
     
+    def __post_init__(self) -> None:
+        # Ensure table_name is always schema.table (strip database if present).
+        # Leave temp tables as-is (e.g., "tempdb..#tmp").
+        if self.table_name:
+            # Skip tempdb pattern
+            if self.table_name.startswith('tempdb..#') or self.table_name.startswith('#'):
+                return
+            parts = self.table_name.split('.')
+            if len(parts) >= 3:
+                self.table_name = '.'.join(parts[-2:])
+    
     def __str__(self) -> str:
         return f"{self.namespace}.{self.table_name}.{self.column_name}"
 
@@ -83,6 +96,8 @@ class ObjectInfo:
     schema: TableSchema
     lineage: List[ColumnLineage] = field(default_factory=list)
     dependencies: Set[str] = field(default_factory=set)  # Tables this object depends on
+    is_fallback: bool = field(default=False)  # Whether this was created by fallback parsing
+    no_output_reason: Optional[str] = field(default=None)  # Reason for no persistent output
 
 
 class SchemaRegistry:
@@ -247,7 +262,8 @@ class ColumnGraph:
     
     def _traverse_upstream(self, column: ColumnNode, max_depth: int, visited: Set[str], current_depth: int = 0) -> List[ColumnEdge]:
         """Recursively traverse upstream dependencies."""
-        if max_depth <= 0 or current_depth >= max_depth:
+        # If max_depth == 0 → unlimited traversal. Only stop when max_depth > 0 and reached.
+        if max_depth > 0 and current_depth >= max_depth:
             return []
         
         column_key = str(column).lower()
@@ -268,7 +284,8 @@ class ColumnGraph:
     
     def _traverse_downstream(self, column: ColumnNode, max_depth: int, visited: Set[str], current_depth: int = 0) -> List[ColumnEdge]:
         """Recursively traverse downstream dependencies."""
-        if max_depth <= 0 or current_depth >= max_depth:
+        # If max_depth == 0 → unlimited traversal. Only stop when max_depth > 0 and reached.
+        if max_depth > 0 and current_depth >= max_depth:
             return []
         
         column_key = str(column).lower()
@@ -311,6 +328,14 @@ class ColumnGraph:
         for obj in objects:
             output_namespace = obj.schema.namespace
             output_table = obj.schema.name
+
+            # Normalize: strip leading '<DB>.' from table name if it matches namespace DB
+            try:
+                ns_db = output_namespace.rsplit('/', 1)[1]
+            except Exception:
+                ns_db = None
+            if ns_db and output_table and output_table.startswith(f"{ns_db}."):
+                output_table = output_table[len(ns_db) + 1:]
             
             for lineage in obj.lineage:
                 # Create output column node
@@ -322,9 +347,19 @@ class ColumnGraph:
                 
                 # Create edges for each input field
                 for input_field in lineage.input_fields:
+                    in_ns = input_field.namespace
+                    in_tbl = input_field.table_name
+                    # Normalize inputs similarly
+                    try:
+                        in_db = in_ns.rsplit('/', 1)[1] if in_ns else None
+                    except Exception:
+                        in_db = None
+                    if in_db and in_tbl and in_tbl.startswith(f"{in_db}."):
+                        in_tbl = in_tbl[len(in_db) + 1:]
+
                     input_column = ColumnNode(
-                        namespace=input_field.namespace,
-                        table_name=input_field.table_name,
+                        namespace=in_ns,
+                        table_name=in_tbl,
                         column_name=input_field.column_name
                     )
                     
